# PAX - the Pythonic API for XML
# developed by Lalo Martins <lalo@laranja.org>
# this file is placed on public domain

from backwards_compatibility import *
from utils import XML

# PAX containers - an Element or, for example, a Document
class Container(object):
    def __init__(self):
        self.children = []

    def __getitem__(self, index):
        return self.children[index]

    def __iter__(self):
        return iter(self.children)

    def __len__(self):
        return len(self.children)

    def __setitem__(self, index, value):
        self.children[index] = value

    def append(self, child):
        self.children.append(child)

    def pack(self):
        "remove Nones, expand tuples"
        result = []
        for child in self:
            if child is None:
                continue
            if type(child) in [type(()), type([])]:
                result.extend(child)
            else:
                result.append(child)
        self.children = result

    def copy(self):
        copy = self.__class__()
        # children is shallow-copied!
        copy.children = self.children[:]
        return copy

# The PAX object
class Element(Container):
    __core_slots = 'ns', 'name', 'attributes', 'children', 'nsdecls', 'raw'
    def __init__(self, ns=u'', name=u'', attributes={}, nsdecls={}, raw=u''):
        self.ns = ns
        self.name = name
        self.attributes = attributes
        self.children = []
        self.nsdecls = nsdecls
        for dprefix, duri in self.nsdecls.items():
            if not attributes.has_key(duri):
                attributes[duri] = {}
        self.raw = raw

    def copy(self):
        copy = self.__class__(self.ns, self.name, {}, self.nsdecls.copy(), self.raw)
        # attributes is copied with one level of depth
        for ns, attrs in self.attributes.items():
            copy.attributes[ns] = attrs.copy()
        # children is shallow-copied!
        copy.children = self.children[:]
        for name, value in self.__dict__.items():
            if name not in self.__core_slots:
                setattr(copy, name, value)
        return copy

    def __repr__(self):
        return '<PAX Element: %s>' % self.__dict__

    def __xml__(self, ns_map):
        if self.nsdecls:
            ns_map = ns_map.copy()
            for prefix, uri in self.nsdecls.items():
                ns_map[uri] = prefix
        prefix = ns_map.get(self.ns, u'')
        if prefix is None: prefix = u''
        if prefix: prefix += u':'
        result = u'<' + prefix + self.name
        for dprefix, duri in self.nsdecls.items():
            result += ' xmlns'
            if dprefix:
                result += ':' + dprefix
            result += '="' + duri + '"'
        for ans, attrs in self.attributes.items():
            aprefix = ns_map.get(ans, u'')
            if aprefix is None: aprefix = u''
            if aprefix: aprefix += u':'
            for aname, avalue in attrs.items():
                if avalue is None: avalue = u''
                if aname:
                    result += ' ' + aprefix + aname + u'="' + avalue + u'"'
        if self.children:
            result += u'>' + XML(self.children, ns_map) + u'</' + prefix + self.name + u'>'
        else:
            result += u' />\n'
        return result
            

    # todo: __str__

# character reference
class CharRef(object):
    def __init__(self, num=0):
        self.num = num

    def __repr__(self):
        return '<PAX Character Reference: %s (%s)>' % (self.num, self.unichr(self.num))

    def __xml__(self, ns_map):
        #FIXME: what is the expected format?
        return u'&#%s;' % self.num

# entity reference
class EntityRef(object):
    def __init__(self, name=u''):
        self.name = name

    def __repr__(self):
        return '<PAX Entity Reference: %s>' % self.name

    def __xml__(self, ns_map):
        return u'&%s;' % self.name

# comment
class Comment(object):
    def __init__(self, text=u''):
        self.text = text

    def __repr__(self):
        return '<PAX Comment: %s>' % self.text

    def __xml__(self, ns_map):
        return u'<!--%s-->' % self.text

# xml/sgml declaration
class Declaration(object):
    def __init__(self, text=u''):
        self.text = text

    def __repr__(self):
        return '<PAX Declaration: %s>' % self.text

    def __xml__(self, ns_map):
        return u'<!%s>' % self.text
